#
# Cybran "Loa" Tactical Missile, mobile unit launcher variant of this missile,
# lower and straighter trajectory. Splits into child projectile if it takes enough
# damage.
#
local HAWKSwarmerMissileProjectile = import('/mods/Hawksunits/lua/Hawkeyeprojectiles.lua').HAWKSwarmerMissileProjectile

HawkSwarmerMissile01 = Class(HAWKSwarmerMissileProjectile) {

    NumChildMissiles = 3,

    OnCreate = function(self)
        HAWKSwarmerMissileProjectile.OnCreate(self)
        self:SetCollisionShape('Sphere', 0, 0, 0, 1)
        self.Split = false
        self:ForkThread(self.MovementThread)
    end,
    
    MovementThread = function(self)
        WaitSeconds(1)
        self:SetTurnRate(360)
    end,
    
    PassDamageData = function(self, damageData)
        HAWKSwarmerMissileProjectile.PassDamageData(self,damageData)
        local launcherbp = self:GetLauncher():GetBlueprint()  
        self.ChildDamageData = table.copy(self.DamageData)
        self.ChildDamageData.DamageAmount = launcherbp.SplitDamage.DamageAmount or 0
        self.ChildDamageData.DamageRadius = launcherbp.SplitDamage.DamageRadius or 1   
    end,      
    
    OnImpact = function(self, targetType, targetEntity)
        local army = self:GetArmy()
        CreateLightParticle( self, -1, army, 3, 7, 'glow_03', 'ramp_fire_11' ) 
        #if I collide with terrain dont split
        if targetType != 'Projectile' then
            self.Split = true
        end
        HAWKSwarmerMissileProjectile.OnImpact(self, targetType, targetEntity)
    end,        
    
    OnDamage = function(self, instigator, amount, vector, damageType)
        if not self.Split and (amount >= self:GetHealth()) then
            self.Split = true        
            local vx, vy, vz = self:GetVelocity()
            local velocity = 8
            local ChildProjectileBP = '/mods/Hawksunits/projectiles/HawkSwarmerMissileSplit01/HawkSwarmerMissileSplit01_proj.bp'
            local angle = (2*math.pi) / self.NumChildMissiles
            local spreadMul = 0.75  # Adjusts the width of the dispersal        

            # Launch projectiles at semi-random angles away from split location
            for i = 0, (self.NumChildMissiles - 1) do
                local xVec = vx + math.sin(i*angle) * spreadMul
                local yVec = vy + math.cos(i*angle) * spreadMul
                local zVec = vz + math.cos(i*angle) * spreadMul 
                local proj = self:CreateChildProjectile(ChildProjectileBP)
                proj:SetVelocity(xVec,yVec,zVec)
                proj:SetVelocity(velocity)
                proj:PassDamageData(self.ChildDamageData)                        
            end
        end
        HAWKSwarmerMissileProjectile.OnDamage(self, instigator, amount, vector, damageType)
    end,
}
TypeClass = HawkSwarmerMissile01

