#****************************************************************************
#**
#**  File     :  /cdimage/units/URA0104/URA0104_script.lua
#**  Author(s):  John Comes, David Tomandl
#**
#**  Summary  :  Cybran GARG Air Transport Script
#**
#**  Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local CAirUnit = import('/lua/cybranunits.lua').CAirUnit
local explosion = import('/lua/defaultexplosions.lua')
local util = import('/lua/utilities.lua')
local Weapon = import('/lua/sim/Weapon.lua').Weapon
local cWeapons = import('/lua/cybranweapons.lua')
local CAAAutocannon = cWeapons.CAAAutocannon
local ADFLaserHighIntensityWeapon = import('/lua/aeonweapons.lua').ADFLaserHighIntensityWeapon

LTC0402 = Class(CAirUnit) {
    Weapons = {
        MauzerCannon = Class(ADFLaserHighIntensityWeapon) {},
    },
    DestroyNoFallRandomChance = 1.1,
    AirDestructionEffectBones = { 'MAIN HULL', 'Right_Attachpoint_Lrg_01', 'Left_Attachpoint_Lrg_01', 'Right_Attachpoint_Lrg_02', 'Left_Attachpoint_Lrg_02', 'Right_Attachpoint_Lrg_03',
                                  'Right_Attachpoint_Med_01', 'Left_Attachpoint_Med_01', 'Right_Attachpoint_Med_02', 'Left_Attachpoint_Med_02',
                                  'Right_Attachpoint_Med_03', 'Left_Attachpoint_Med_03', 'Right_Attachpoint_Med_04', 'Left_Attachpoint_Med_04',
                                  'Right_Attachpoint_05', 'Left_Attachpoint_05', 'Right_Attachpoint_06', 'Left_Attachpoint_06', },

    BeamExhaustIdle = '/effects/emitters/missile_exhaust_fire_beam_05_emit.bp',
    BeamExhaustCruise = '/effects/emitters/missile_exhaust_fire_beam_04_emit.bp',
    

    # When one of our attached units gets killed, detach it
    OnAttachedKilled = function(self, attached)
        attached:DetachFrom()
    end,

    OnKilled = function(self, instigator, type, overkillRatio)
        CAirUnit.OnKilled(self, instigator, type, overkillRatio)
        # TransportDetachAllUnits takes 1 bool parameter. If true, randomly destroys some of the transported
        # units, otherwise successfully detaches all.
        self:TransportDetachAllUnits(true)
    end,

    # Override air destruction effects so we can do something custom here
    CreateUnitAirDestructionEffects = function( self, scale )
        self:ForkThread(self.AirDestructionEffectsThread, self )
    end,

    AirDestructionEffectsThread = function( self )
        local numExplosions = math.floor( table.getn( self.AirDestructionEffectBones ) * 0.5 )
        for i = 0, numExplosions do
            explosion.CreateDefaultHitExplosionAtBone( self, self.AirDestructionEffectBones[util.GetRandomInt( 1, numExplosions )], 0.5 )
            WaitSeconds( util.GetRandomFloat( 0.2, 0.9 ))
        end
    end,
}

TypeClass = LTC0402

