#****************************************************************************
#**
#**  File     :  /cdimage/units/UES0302/UES0302_script.lua
#**  Author(s):  John Comes, David Tomandl, Jessica St. Croix
#**
#**  Summary  :  UEF Battleship Script
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local TSeaUnit = import('/lua/terranunits.lua').TSeaUnit
local WeaponsFile = import('/lua/terranweapons.lua')
local weaponfile2 = import('/mods/Hawksunits/lua/Hawkeyeweapons.lua')
local AALaserweapon = weaponfile2.AALaserweapon
local TAMPhalanxWeapon = WeaponsFile.TAMPhalanxWeapon
local TIFCruiseMissileLauncher = WeaponsFile.TIFCruiseMissileLauncher
local TDFGaussCannonWeapon = WeaponsFile.TDFGaussCannonWeapon
local TDFLightPlasmaCannonWeapon = WeaponsFile.TDFLightPlasmaCannonWeapon

UES0302 = Class(TSeaUnit) {


    Weapons = {
        RightAAGun01 = Class(TDFLightPlasmaCannonWeapon) {},
        RightAAGun02 = Class(TDFLightPlasmaCannonWeapon) {},
        LeftAAGun01 = Class(TDFLightPlasmaCannonWeapon) {},
        LeftAAGun02 = Class(TDFLightPlasmaCannonWeapon) {},
        Turret01 = Class(TDFGaussCannonWeapon) {},
        Turret02 = Class(TDFGaussCannonWeapon) {},
        Turret03 = Class(TDFGaussCannonWeapon) {},
        Turret04 = Class(TDFGaussCannonWeapon) {},
        Turret05 = Class(TDFGaussCannonWeapon) {},
        Turret06 = Class(TDFGaussCannonWeapon) {},
        Turret07 = Class(TDFGaussCannonWeapon) {},
        Turret08 = Class(TDFGaussCannonWeapon) {},
        Annihilator = Class(TDFLightPlasmaCannonWeapon) {},
        CruiseMissile = Class(TIFCruiseMissileLauncher) {
                CurrentRack = 1,
           
                PlayFxMuzzleSequence = function(self, muzzle)
                    local bp = self:GetBlueprint()
                    self.Rotator = CreateRotator(self.unit, bp.RackBones[self.CurrentRack].RackBone, 'y', nil, 90, 90, 90)
                    muzzle = bp.RackBones[self.CurrentRack].MuzzleBones[1]
                    self.Rotator:SetGoal(90)
                    TIFCruiseMissileLauncher.PlayFxMuzzleSequence(self, muzzle)
                    WaitFor(self.Rotator)
                    WaitSeconds(1)
                end,
                
                CreateProjectileAtMuzzle = function(self, muzzle)
                    muzzle = self:GetBlueprint().RackBones[self.CurrentRack].MuzzleBones[1]
                    if self.CurrentRack >= 8 then
                        self.CurrentRack = 1
                    else
                        self.CurrentRack = self.CurrentRack + 1
                    end
                    TIFCruiseMissileLauncher.CreateProjectileAtMuzzle(self, muzzle)
                end,
                
                PlayFxRackReloadSequence = function(self)
                    WaitSeconds(1)
                    self.Rotator:SetGoal(0)
                    WaitFor(self.Rotator)
                    self.Rotator:Destroy()
                    self.Rotator = nil
                end,
            },
        RightPhalanxGun01 = Class(TAMPhalanxWeapon) {
            PlayFxWeaponUnpackSequence = function(self)
                if not self.SpinManip then 
                    self.SpinManip = CreateRotator(self.unit, 'Right_Turret02_Barrel', 'z', nil, 270, 180, 60)
                    self.unit.Trash:Add(self.SpinManip)
                end
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(270)
                end
                TAMPhalanxWeapon.PlayFxWeaponUnpackSequence(self)
            end,

            PlayFxWeaponPackSequence = function(self)
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(0)
                end
                TAMPhalanxWeapon.PlayFxWeaponPackSequence(self)
            end,
            
        },
        LeftPhalanxGun01 = Class(TAMPhalanxWeapon) {
            PlayFxWeaponUnpackSequence = function(self)
                if not self.SpinManip then 
                    self.SpinManip = CreateRotator(self.unit, 'Left_Turret02_Barrel', 'z', nil, 270, 180, 60)
                    self.unit.Trash:Add(self.SpinManip)
                end
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(270)
                end
                TAMPhalanxWeapon.PlayFxWeaponUnpackSequence(self)
            end,

            PlayFxWeaponPackSequence = function(self)
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(0)
                end
                TAMPhalanxWeapon.PlayFxWeaponPackSequence(self)
            end,

        },
        FrontTurret01 = Class(TDFGaussCannonWeapon) {
            FxMuzzleFlash = {
                '/effects/emitters/cannon_muzzle_fire_01_emit.bp',
                '/effects/emitters/gauss_cannon_muzzle_flash_02_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_03_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_04_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_05_emit.bp',
                '/effects/emitters/cannon_muzzle_water_shock_01_emit.bp',
            },
            FxMuzzleFlashScale = 0.8,
            },
        FrontTurret02 = Class(TDFGaussCannonWeapon) {
            FxMuzzleFlash = {
                '/effects/emitters/cannon_muzzle_fire_01_emit.bp',
                '/effects/emitters/gauss_cannon_muzzle_flash_02_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_03_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_04_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_05_emit.bp',
                '/effects/emitters/cannon_muzzle_water_shock_01_emit.bp',
            },
            FxMuzzleFlashScale = 0.8,
            },
        BackTurret = Class(TDFGaussCannonWeapon) {
            FxMuzzleFlash = {
                '/effects/emitters/cannon_muzzle_fire_01_emit.bp',
                '/effects/emitters/gauss_cannon_muzzle_flash_02_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_03_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_04_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_05_emit.bp',
                '/effects/emitters/cannon_muzzle_water_shock_01_emit.bp',
            },
            FxMuzzleFlashScale = 0.8,
            },
        BackTurret01 = Class(TDFGaussCannonWeapon) {
            FxMuzzleFlash = {
                '/effects/emitters/cannon_muzzle_fire_01_emit.bp',
                '/effects/emitters/gauss_cannon_muzzle_flash_02_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_03_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_04_emit.bp',
                '/effects/emitters/cannon_muzzle_smoke_05_emit.bp',
                '/effects/emitters/cannon_muzzle_water_shock_01_emit.bp',
            },
            FxMuzzleFlashScale = 0.8,
            },
    },

    OnStopBeingBuilt = function(self,builder,layer)
        TSeaUnit.OnStopBeingBuilt(self,builder,layer)
        self.Trash:Add(CreateRotator(self, 'Spinner01', 'y', nil, -45))
        self.Trash:Add(CreateRotator(self, 'Spinner02', 'y', nil, 90))
    end,

    OnKilled = function(self, inst, type, okr)
        self.Trash:Destroy()
        self.Trash = TrashBag()
	TSeaUnit.OnKilled(self, inst, type, okr)
    end,
}

TypeClass = UES0302